;;; -*-Emacs-Lisp-*- Tea mode
;;; Copyright Olin Shivers (1988)
;;; Please imagine a long, tedious, legalistic 5-page gnu-style copyright
;;; notice appearing here to the effect that you may use this code any
;;; way you like, as long as you don't charge money for it, remove this
;;; notice, or hold me liable for its results.

;;; Written by Olin Shivers (shivers@cs.cmu.edu) 10/88.
;;; Please send me bug reports, bug fixes, and extensions, so that I can
;;; merge them into the master source.

;;; This file tunes scheme and inferior-scheme modes for T.
;;; This is a fairly simple operation, and consists of:
;;; 1. Adding T-MODE to the list of modes that are recognised by 
;;;    scheme-load-file and scheme-compile-file as marking source files.
;;; 2. Changing the scheme-compile-exp-command string to
;;;    "(orbit '%s)"
;;; 3. Tweaking the scheme syntax table just a bit.
;;; 5. Specifying indentation information for a few T specific special forms,
;;;    and turning off the scheme-mit-dialect flag.
;;; 3. Having commands TEA and RUN-TEA that are just like SCHEME and
;;;    RUN-SCHEME except that (a) that the name of the program to run isn't
;;;    taken from the variable SCHEME-PROGRAM-NAME, it's taken from
;;;    TEA-PROGRAM-NAME, and (b) the mode hook isn't INFERIOR-SCHEME-MODE-HOOK
;;;    but INFERIOR-T-MODE-HOOK.
;;; 4. Having a mode T-MODE, that is exactly equal to SCHEME-MODE, except
;;;    that the hooks from T-MODE-HOOK are run instead of SCHEME-MODE-HOOK.
 
;;; This is the minimal-delta approach to a T mode: we just mutate Scheme
;;; mode. Note, for example, that there is no "inferior-t mode."
;;; This means you can run exactly *one* scheme and/or T
;;; process at a time; you *can't* run a scheme process *and* a T process.
;;; I don't think this is a problem.

;;; This stuff uses the scheme-in-a-buffer stuff defined in cmuscheme.el and
;;; comint.el; the commands provided are extensively documented in the
;;; source text for these files.

;; YOUR .EMACS FILE
;;=============================================================================
;; Some suggestions for your .emacs file.
;;
;; ; If tea.el lives in some non-standard directory, you must tell emacs
;; ; where to get it. This may or may not be necessary.
;; (setq load-path (cons (expand-file-name "~jones/lib/emacs") load-path))
;;
;; ; Autoload run-tea and t-mode from file tea.el
;; (autoload 'run-tea "tea"
;;           "Run an inferior T process."
;;           t)
;;
;; (autoload 't-mode "tea"
;;           "Major mode for editing T source. Just Scheme mode, tuned a bit."
;;           t)
;;
;; ; Files ending in ".t" are T source, so put their buffers in t-mode.
;; (setq auto-mode-alist
;;       (cons '("\\.t$" . t-mode) 
;;	       auto-mode-alist))   
;;
;; ; Define C-c C-t to run my favorite command in inferior Scheme mode:
;; (setq tea-load-hook
;;       '((lambda () (define-key inferior-scheme-mode-map "\C-c\C-t"
;;                                'favorite-cmd))))

;; ETAGS
;;=============================================================================
;; A suggestion for modifying the etags program so that it knows about T.
;; You should modify the few lines that allow etags to conclude that
;; files that end with ".t" are lisp or scheme source code.  
;; Find a line that looks like
;; /* .scm or .sm or .scheme implies scheme source code */
;; and add a 
;;	     !strcmp (cp + 1, "t") ||
;; suffix check to the following clauses that check filename suffixes.
;; This is already done for some versions of etags. Have a look, or try it 
;; & see.

(require 'cmuscheme)	; For inferior-scheme mode
(provide 'tea)

; File buffers in t-mode contain scheme source
(if (not (memq 't-mode scheme-source-modes))  
    (setq scheme-source-modes (cons 't-mode scheme-source-modes)))

(setq scheme-compile-exp-command "(orbit '%s)")

;; This adds [] and {} as matching delimiters. So emacs will treat #[Char 0]
;; or #{Procedure 1 ADD} as an s-exp with a quote sign in front.
(modify-syntax-entry ?[ "(]" scheme-mode-syntax-table)
(modify-syntax-entry ?] ")[" scheme-mode-syntax-table)
(modify-syntax-entry ?{ "(}" scheme-mode-syntax-table)
(modify-syntax-entry ?} "){" scheme-mode-syntax-table)

;; Modify indenting for T.
(setq scheme-mit-dialect nil) ; Give me a break.
(put 'labels 'scheme-indent-hook 1)
(put 'block 'scheme-indent-hook 0)
(put 'block0 'scheme-indent-hook 0)
(put 'object 'scheme-indent-hook 1)
(put 'lset 'scheme-indent-hook 1)
(put 'xcase 'scheme-indent-hook 1)
(put 'select 'scheme-indent-hook 1)
(put 'xselect 'scheme-indent-hook 1)
(put 'iterate 'scheme-indent-hook 2)
(put 'cond 'scheme-indent-hook 0)
(put 'xcond 'scheme-indent-hook 0)
(put 'catch 'scheme-indent-hook 1)
(put 'bind 'scheme-indent-hook 1)
(put 'define-operation 'scheme-indent-hook 1)
(put 'operation 'scheme-indent-hook 1)
(put 'object 'scheme-indent-hook 1)
(put 'join 'scheme-indent-hook 0)
(put 'destructure 'scheme-indent-hook 1)
(put 'destructure* 'scheme-indent-hook 1)
(put 'define-integrable 'scheme-indent-hook 1)
(put 'define-constant 'scheme-indent-hook 1)
(put 'define-syntax 'scheme-indent-hook 1)
(put 'let-syntax 'scheme-indent-hook 1)
(put 'define-local-syntax 'scheme-indent-hook 1)
(put 'macro-expander 'scheme-indent-hook 1)
(put 'with-open-streams 'scheme-indent-hook 1)
(put 'with-open-ports 'scheme-indent-hook 1)
(put 'with-input-from-string 'scheme-indent-hook 1)
(put 'with-output-to-string 'scheme-indent-hook 1)
(put 'with-output-width-string 'scheme-indent-hook 1)
(put 'receive 'scheme-indent-hook 1)
(put 'receive-values 'scheme-indent-hook 1)

(defvar tea-program-name "t"
  "*Name of the T program to run when RUN-TEA is called.")

(defun run-tea (arg)
  "Run an inferior T process, input and output via buffer *scheme*.
With argument, it asks for a command line.  Take the program name from the
variable tea-program-name. Runs the hooks from inferior-t-hook (after
comint-mode-hook is run).
\(Type \\[describe-mode] in the process buffer for a list of commands.)"
  (interactive "P")
  (if arg (call-interactively 'tea)
      (tea "")))

(defun tea (arg)
  "Like run-tea, except prompts for a command line."
  (interactive "sExtra arguments to tea: ")
  (let ((scheme-program-name tea-program-name)	   ; Lexical scope loses!
	(inferior-scheme-mode-hook (function (lambda (); Dynamic binding rules!
				     (run-hooks 'inferior-t-mode-hook)))))
    (scheme arg)))


(defvar inferior-t-mode-hook nil
  "*Hook for customising inferior-scheme mode when invoked via run-tea or tea")


(defun t-mode ()
  "Major mode for editing T code. Basically, it's just scheme mode, except 
for the entry hook.
Commands:
Delete converts tabs to spaces as it moves back.
Blank lines separate paragraphs.  Semicolons start comments.
\\{scheme-mode-map}
Entry to this mode runs the hooks on t-mode-hook.
If you accidentally suspend your process, use \\[comint-continue-subjob]
to continue it."
  (interactive)
  (kill-all-local-variables)
  (use-local-map scheme-mode-map)
  (setq major-mode 't-mode)
  (setq mode-name "T")
  (scheme-mode-variables)
  (run-hooks t-mode-hook))

(defvar t-mode-hook nil
  "*Hook for customising T mode.")

;;; Do the user's customisation...
(defvar tea-load-hook nil
  "This hook is run when tea is loaded in.
This is a good place to put keybindings.")
	
(run-hooks 'tea-load-hook)
